@@ -11,7 +11,7 @@ use RDF::Trine::Graph;
 use Scalar::Util qw/blessed/;
 
 use base 'Test::Builder::Module';
-our @EXPORT = qw/are_subgraphs is_rdf is_valid_rdf isomorph_graphs has_subject has_predicate has_object_uri has_uri hasnt_uri has_literal pattern_target pattern_ok/;
+our @EXPORT = qw/are_subgraphs is_rdf is_valid_rdf isomorph_graphs has_subject has_predicate has_object_uri has_uri hasnt_uri has_literal pattern_target pattern_ok pattern_fail/;
 
 
 =head1 NAME
@@ -20,11 +20,11 @@ Test::RDF - Test RDF data for content, validity and equality, etc.
 
 =head1 VERSION
 
-Version 1.00
+Version 1.15_1
 
 =cut
 
-our $VERSION = '1.00';
+our $VERSION = '1.15_1';
 
 
 =head1 SYNOPSIS
@@ -43,6 +43,7 @@ our $VERSION = '1.00';
  has_literal($string, $language, $datatype, $model, 'Literal is found');
  pattern_target($model);
  pattern_ok($pattern, '$pattern found in $model');
+ pattern_fail($pattern, '$pattern not found in $model');
 
 =head1 DESCRIPTION
 
@@ -50,7 +51,8 @@ This Perl module, Test::RDF, provides tools for testing code which
 deals with RDF. It can test RDF for validity, check if two RDF graphs
 are the same, or subgraphs of each other, if a URI is or is not in a
 dataset, if it has certain subjects, predicates, objects or
-literals. It can also test to see if a full pattern is present.
+literals. It can also test to see if a full pattern is present or
+absent.
 
 
 =head1 EXPORT
@@ -178,7 +180,9 @@ of the statements given in the model given as second argument.
 
 sub has_subject {
   my ($uri, $model, $name) = @_;
-  my $count = $model->count_statements(RDF::Trine::Node::Resource->new($uri), undef, undef);
+  my $resource = _resource_uri_checked($uri, $name);
+  return $resource unless ($resource);
+  my $count = $model->count_statements($resource, undef, undef);
   return _single_uri_tests($count, $name);
 }
 
@@ -192,7 +196,9 @@ of the statements given in the model given as second argument.
 
 sub has_predicate {
   my ($uri, $model, $name) = @_;
-  my $count = $model->count_statements(undef, RDF::Trine::Node::Resource->new($uri), undef);
+  my $resource = _resource_uri_checked($uri, $name);
+  return $resource unless ($resource);
+  my $count = $model->count_statements(undef, $resource, undef);
   return _single_uri_tests($count, $name);
 }
 
@@ -205,7 +211,9 @@ of the statements given in the model given as second argument.
 
 sub has_object_uri {
   my ($uri, $model, $name) = @_;
-  my $count = $model->count_statements(undef, undef, RDF::Trine::Node::Resource->new($uri));
+  my $resource = _resource_uri_checked($uri, $name);
+  return $resource unless ($resource);
+  my $count = $model->count_statements(undef, undef, $resource);
   return _single_uri_tests($count, $name);
 }
 
@@ -267,9 +275,11 @@ the statements given in the model given as second argument.
 sub has_uri {
   my ($uri, $model, $name) = @_;
   my $test = __PACKAGE__->builder;
-  if ($model->count_statements(undef, undef, RDF::Trine::Node::Resource->new($uri)) > 0
-      || $model->count_statements(undef, RDF::Trine::Node::Resource->new($uri), undef) > 0
-      || $model->count_statements(RDF::Trine::Node::Resource->new($uri), undef, undef) > 0) {
+  my $resource = _resource_uri_checked($uri, $name);
+  return $resource unless ($resource);
+  if ($model->count_statements(undef, undef, $resource) > 0
+      || $model->count_statements(undef, $resource, undef) > 0
+      || $model->count_statements($resource, undef, undef) > 0) {
     $test->ok( 1, $name );
     return 1;
   } else {
@@ -290,9 +300,13 @@ of the statements given in the model given as second argument.
 sub hasnt_uri {
   my ($uri, $model, $name) = @_;
   my $test = __PACKAGE__->builder;
-  if ($model->count_statements(undef, undef, RDF::Trine::Node::Resource->new($uri)) > 0
-      || $model->count_statements(undef, RDF::Trine::Node::Resource->new($uri), undef) > 0
-      || $model->count_statements(RDF::Trine::Node::Resource->new($uri), undef, undef) > 0) {
+  my $resource;
+  eval {
+	  $resource = RDF::Trine::Node::Resource->new($uri);
+  };
+  if (($resource) && ($model->count_statements(undef, undef, $resource) > 0
+      || $model->count_statements(undef, $resource, undef) > 0
+      || $model->count_statements($resource, undef, undef)) > 0) {
     $test->ok( 0, $name );
     $test->diag('Matching URIs found in model');
     return 0;
@@ -317,6 +331,26 @@ sub _single_uri_tests {
   }
 }
 
+sub _resource_uri_checked {
+	my ($uri, $name) = @_;
+	my $resource;
+	eval {
+		$resource = RDF::Trine::Node::Resource->new($uri);
+	};
+	if ( my $error = $@ ) {
+		my $test = __PACKAGE__->builder;
+		local $Test::Builder::Level = $Test::Builder::Level + 1;
+		$test->ok( 0, $name );
+		$test->diag("No matching URIs found in model");
+		return 0;
+	}
+	return $resource;
+}
+
+		
+
+
+
 =head2 pattern_target
 
 Tests that the object passed as its parameter is an RDF::Trine::Model or
@@ -353,6 +387,10 @@ a list of RDF::Trine::Statement objects.
 
 B<Note:> C<pattern_target> must have been tested before any C<pattern_ok> tests.
 
+=head2 pattern_fail
+
+The same as above, but tests if the pattern returns no results instead.
+
 =cut
 
 { # scope for $target
@@ -375,6 +413,7 @@ B<Note:> C<pattern_target> must have been tested before any C<pattern_ok> tests.
       return 0;
     }
   }
+
   sub pattern_ok {
     my $message = undef;
     $message = pop @_ if !ref $_[-1];
@@ -390,11 +429,14 @@ B<Note:> C<pattern_target> must have been tested before any C<pattern_ok> tests.
     my $pattern = (blessed($_[0]) and $_[0]->isa('RDF::Trine::Pattern'))
                 ? $_[0]
                 : RDF::Trine::Pattern->new(@_);
+	 my $s = RDF::Trine::Serializer::Turtle->new();
+
     my $iter    = $target->get_pattern($pattern);
-    while (my $row = $iter->next) {
+    if ($iter->materialize->length > 0) {
       $test->ok(1, $message);
       return 1;
     }
+	 # The test result is now known, return diagnostics
 	 my $noreturns;
 	 foreach my $triple ($pattern->triples) {
 		 my @triple;
@@ -416,6 +458,33 @@ B<Note:> C<pattern_target> must have been tested before any C<pattern_ok> tests.
 	 }
     return 0;
   }
+
+  sub pattern_fail {
+    my $message = undef;
+    $message = pop @_ if !ref $_[-1];
+    unless (defined $message and length $message) {
+      $message = "Pattern doesn't match";
+    }
+    my $test = __PACKAGE__->builder;
+    unless (blessed($target)) {
+      $test->ok(0, $message);
+      $test->diag("No target defined for pattern match. Call pattern_target test first.");
+      return 0;
+    }
+    my $pattern = (blessed($_[0]) and $_[0]->isa('RDF::Trine::Pattern'))
+                ? $_[0]
+                : RDF::Trine::Pattern->new(@_);
+    my $iter = $target->get_pattern($pattern)->materialize;
+
+    if ($iter->length == 0) {
+      $test->ok(1, $message);
+      return 1;
+    }
+	 # The test result is now known, return diagnostics
+    $test->ok(0, $message);
+	 $test->diag("These triples had results:\n" . $iter->as_string);
+    return 0;
+  }
 } # /scope for $target
 
 
@@ -476,7 +545,7 @@ Toby Inkster has submitted the pattern_* functions.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010 ABC Startsiden AS and 2010-2012 Kjetil Kjernsmo.
+Copyright 2010 ABC Startsiden AS and 2010-2013 Kjetil Kjernsmo.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
